import acm.program.*;
import acm.graphics.*;
import acm.util.*;
import java.awt.event.*;
import java.awt.*;
import java.io.*;

/**
 * A program that creates a collection of objects on the screen, then lets
 * the user click and drag them around.
 */
public class SuchADrag extends GraphicsProgram {
	public void run() {
		addObjects();
		addMouseListeners();
	}
	
	/* The object which has been selected for dragging. */
	private GObject selectedObject;
	
	/* The mouse's last position. */
	private double lastX, lastY;
	
	/**
	 * Selects the object under the mouse cursor when the mouse is pressed.
	 * If nothing is found, that's okay - we'll set selectedObject to null.
	 */
	public void mousePressed(MouseEvent e) {
		selectedObject = getElementAt(e.getX(), e.getY());
		lastX = e.getX();
		lastY = e.getY();
	}
	
	/**
	 * Repositions the dragged object to the mouse's location when the mouse
	 * is moved.
	 */
	public void mouseDragged(MouseEvent e) {
		/* If there is something to drag at all, go move it. */
		if (selectedObject != null) {
			double dx = e.getX() - lastX;
			double dy = e.getY() - lastY;
			
			selectedObject.move(dx, dy);
			
			lastX = e.getX();
			lastY = e.getY();
		}
	}

	/**
	 * Adds the "magnets" to the display.
	 */
	private void addObjects() {
		try {
			/* Open the file for reading. */
			BufferedReader br = new BufferedReader(new FileReader("words.txt"));
			
			/* Process the file by adding one magnet per line. */
			while (true) {
				String line = br.readLine();
				if (line == null) break;
				
				addMagnet(line);
			}
			
			br.close();
		} catch (IOException e) {
			println(":-(");
		}
	}
	
	/**
	 * Adds a randomly-positioned "magnet" to the display with the given
	 * label.
	 * 
	 * @param text The text to use for the magnet.
	 */
	private void addMagnet(String text) {
		GLabel label = new GLabel(text);
		label.setFont("DejaVuSerif-24");
		label.setColor(Color.BLUE);
		
		/* Randomly position the label. Due to the way that the label draws,
		 * we need to factor in the text ascent and descent when computing the
		 * y coordinate.
		 */
		RandomGenerator rgen = RandomGenerator.getInstance();
		double x = rgen.nextDouble(0, getWidth() - label.getWidth());
		double y = rgen.nextDouble(label.getAscent(), getHeight() - label.getDescent());
		
		label.setLocation(x, y);
		add(label);
	}
}
